﻿using IRaCIS.Core.Domain.Models;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Infra.EFCore.Common;
using Microsoft.AspNetCore.Mvc.Diagnostics;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.EntityFrameworkCore.Diagnostics;
using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;


namespace IRaCIS.Core.Infra.EFCore;

public class AuditEntityInterceptor(IUserInfo _userInfo) : SaveChangesInterceptor
{

    /// <summary>
    /// 在事务提交之前执行
    /// </summary>
    /// <param name="eventData"></param>
    /// <param name="result"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public override ValueTask<InterceptionResult<int>> SavingChangesAsync(DbContextEventData eventData,
        InterceptionResult<int> result, CancellationToken cancellationToken = default)
    {
        //审计时间
        AuditEntities(eventData.Context);

        //IRC稽查  放在savechange 之前 不影响之前的逻辑
        IRCDataInspection(eventData.Context);

        return base.SavingChangesAsync(eventData, result, cancellationToken);
    }
    public override InterceptionResult<int> SavingChanges(DbContextEventData eventData, InterceptionResult<int> result)
    {
        AuditEntities(eventData.Context);

        return base.SavingChanges(eventData, result);
    }
    public void AuditEntities(DbContext? context)
    {
        if (context == null) return;

        // 获取所有更改，删除，新增的实体，但排除审计实体（避免死循环）
        foreach (var entry in context.ChangeTracker.Entries().Where(u => (u.State == EntityState.Modified || u.State == EntityState.Added))
                                                             .Where(x => !typeof(DataInspection).IsAssignableFrom(x.Entity.GetType())).ToList())
        {
            switch (entry.State)
            {

                case EntityState.Deleted:

                    break;
                case EntityState.Modified:

                    if (entry.Entity is IAuditUpdate updateEntity1)
                    {
                        updateEntity1.UpdateTime = DateTime.Now;
                        updateEntity1.UpdateUserId = _userInfo.Id;
                    }

                    if (entry.Entity is ISoftDelete softDelete)
                    {
                        if (softDelete.IsDeleted)
                        {
                            softDelete.DeleteUserId = _userInfo.Id;
                            softDelete.DeletedTime = DateTime.Now;
                        }
                        else
                        {
                            softDelete.DeletedTime = null;
                        }
                    }

                    break;
                //添加的时候，更新审计字段也赋值
                case EntityState.Added:


                    if (entry.Entity is IAuditAdd addEntity)
                    {
                        if (addEntity.CreateTime == default(DateTime))
                        {
                            addEntity.CreateTime = DateTime.Now;
                        }

                        addEntity.CreateUserId = _userInfo.Id;
                    }

                    if (entry.Entity is IAuditUpdate updateEntity)
                    {
                        updateEntity.UpdateTime = DateTime.Now;
                        updateEntity.UpdateUserId = _userInfo.Id;
                    }

                    if (entry.Entity is IAuditAddWithUserName addEntity3)
                    {
                        if (addEntity3.CreateTime == default(DateTime))
                        {
                            addEntity3.CreateTime = DateTime.Now;

                        }

                        addEntity3.CreateUserId = _userInfo.Id;
                        addEntity3.CreateUser = _userInfo.RealName;
                    }
                    break;
            }
        }
    }

    public void IRCDataInspection(DbContext? context)
    {
        //触发器里面提交事务 业务方法里面提交事务 会记录两次
        var inspectionGeneralIdList = context.ChangeTracker.Entries().Where(t => typeof(DataInspection).IsAssignableFrom(t.Entity.GetType())).Select(t => ((DataInspection)t.Entity).GeneralId).ToList();

        var entities = context.ChangeTracker.Entries().Where(u => (u.State == EntityState.Modified || u.State == EntityState.Deleted || u.State == EntityState.Added))
            .Where(t => !typeof(DataInspection).IsAssignableFrom(t.Entity.GetType()) && !inspectionGeneralIdList.Contains(((Entity)t.Entity).Id))
            .ToList();
        AuditingData auditingData = new AuditingData((IRaCISDBContext)context, _userInfo);


        if (entities.Count > 0)
        {
            auditingData.InsertAddEntitys(entities).GetAwaiter().GetResult();

        }
    }
}